/**
 * Licensed Materials - Property of IBM
 *
 * IBM Cognos Products: BI
 *
 * Copyright IBM Corp. 2020
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/**
 	* @file Lasso selection view
 	*/

	 'use strict';
	 define([
		 'jquery',
		 'dashboard-core/js/lib/@waca/core-client/js/core-client/ui/View',
		 'dashboard-core/js/lib/@waca/core-client/js/core-client/utils/BrowserUtils',
		 '@waca/dashboard-common/js/utils/EventChainLocal'
	 ],function($, BaseView, BrowserUtils, EventChainLocal) {
	 
		 const EXTENSION_ID = 'com.ibm.bi.dashboard.live-lassoView.polygon';
	 
		 const keyCodes = {
			 LEFT_CLICK:1,
			 ESCAPE: 27
		 };
	 
		 const POINTER_EVENTS = {
			 DOWN: 'mousedown touchstart',
			 MOVE: 'mousemove touchmove',
			 UP: 'mouseup touchend',
			 LEAVE: 'mouseleave',
			 CLICK: 'click touch',
			 DLBCLICK: 'dblclick'
		 };
	 
		 const CURSOR = {
			 DEFAULT: 'url(dashboard-analytics/images/lasso-polygon.svg), auto',
			 CLOSED: 'url(dashboard-analytics/images/lasso-polygon-closed.svg), auto'
		 };
	 
		 const HIDE_DURATION = 2000;
	 
		 //Closing the polygon parameters
		 const MIN_DISTANCE = 10;
		 const MIN_NUMBER_OF_POINTS = 2;
	 
		 var LassoSelectView = BaseView.extend({
	 
			 id: EXTENSION_ID,
	 
			 /**
					  * @param {options} set of initial properties
					  *
					  */
			 init: function($target, cordinateResolver, isDragging) {
				 BaseView.inherited('init', this, arguments);
	 
				 this.$target = $target;
				 this.targetNode = this.$target.get(0);
				 this.$body = $(document.body);
				 this.cordinateResolver = cordinateResolver;
				 this._isDragging = isDragging;
	 
				 this.handlers = {
					 mousemove: this._getPipedHandler(this._handleMouseMove),
					 mouseup: this._getPipedHandler(this._handleMouseUp),
					 mousedown: this._getPipedHandler(this._handleMouseDown),
					 keyup: this._getPipedHandler(this._handleKeyup),
					 mouseleave: this._getPipedHandler(this._handleMouseLeave),
					 dblclick: this._getPipedHandler(this._handleDblClick)
				 };
	 
				 if(!this._isIE()){
					 this._configCursor();
				 }
	 
				 this.$body.keyup(this.handlers.keyup);
				 this.$target
					 .on(POINTER_EVENTS.LEAVE, this.handlers.mouseleave)
					 .on(POINTER_EVENTS.MOVE, this.handlers.mousemove)
					 .on(POINTER_EVENTS.DLBCLICK, this.handlers.dblclick)
					 .on(POINTER_EVENTS.UP, this.handlers.mouseup)
					 .on(POINTER_EVENTS.DOWN, this.handlers.mousedown);

	 
				 this._points = [];
			 },
	 
			 _handleDblClick: function(event){
				 this._points.push(this._points[0]);
				 this._render();
				 this._done(event);
			 },
	 
			 _done: function(event){
				 this.lastLine = null;
				 this.trigger('lassoSelect:done', { event: event, cords: this._getCords() });
				 this.$svg.children().each(function (index, child){
					$(child).hide(HIDE_DURATION, function() {$(this).remove(); })
				 });
				 this._points.length = 0;
			 },
	 
			 /**
				  * Mouse move event handler
				  * @param {Object} event Jquery event object
				  */
			 _handleMouseMove: function(event){
	 
				 this._updateCursor(event);
				 if(this.lastLine){
					 const rect  = this.targetNode.getBoundingClientRect();
					 const top = rect.top;
					 const left = rect.left;
					 this.lastLine.setAttribute( 'x2' , event.clientX - left);
					 this.lastLine.setAttribute( 'y2' , event.clientY - top);
				 }
			 },
	 
			 /**
				  * Mouse up event handler
				  * @param {Object} event Jquery event object
				  */
			 _handleMouseUp: function(event){
	 
				 let points = { x: event.clientX, y: event.clientY };
				 let done = false;
				 if( this._canAutoClosePolygon(event)){
					 points = this._points[0];
					 done = true;
				 }
				 this._points.push(points);
				 this._render();
	 
				 if(done){
					 this._done(event);
				 }
			 },

			  /**
			  * Mouse down event handler
			  * @param {Object} event Jquery event object
			  */
			 _handleMouseDown: function(){},
	 
			 /**
					  * Handle key up event
					  * @param {Object} event Jquery event object
					  */
			 _handleKeyup: function(e){
				 if(e.keyCode === keyCodes.ESCAPE){
					 this.trigger('lassoSelect:cancel', {});
				 }
			 },
	 
			 _handleMouseLeave: function(event){
				 this._updateCursor(event, false);
			 },
	 
			 _render: function(){
	 
				 if(this.$svg){
					 this.$svg.remove();
				 }
	 
				
				 const rect  = this.targetNode.getBoundingClientRect();
				 const top = rect.top;
				 const left = rect.left;
				 const width = rect.width;
				 const height = rect.height;

				 const lines = this._points.map(function (point, index){
					 const line =  document.createElement('line');
					 const x1 =  Math.abs(point.x - left);
					 const y1 = Math.abs(point.y - top);
					 const attributes = { x1:x1, y1:y1, x2: x1, y2: y1 };
					 if(this._points[index + 1]){
						 attributes['x2'] = Math.abs(this._points[index + 1].x - left);
						 attributes['y2'] = Math.abs(this._points[index + 1].y - top);
					 }
					 Object.keys(attributes).forEach(function(name){
						line.setAttribute(name, attributes[name])
					 });
					 return line;
				 }.bind(this));
	 
				 const svg = document.createElement('svg');
				 lines.forEach(function(line){
					svg.appendChild(line)
				 });
	 
				 svg.setAttribute('id', this.id);
				 $(svg).css({ left:left, top:top, width:width, height:height });
	 
				 this.$body.prepend(svg.outerHTML);
				 this.$svg = $(document.getElementById(this.id));
				 this.lastLine = this.$svg.children().last().get(0);
			 },
	 
			 _distance: function(pointA, pointB){
				 if(!pointA || !pointB){
					 return -1;
				 }
	 
				 const deltaX = pointB.x - pointA.x;
				 const deltaY = pointB.y - pointA.y;
				 return Math.sqrt( deltaX * deltaX + deltaY * deltaY );
			 },
	 
			 remove: function(){
	 
				 this.$body.off(POINTER_EVENTS.UP, this.handlers.mouseup)
					 .off('keyup', this.handlers.keyup);
	 
				 this.$target.off(POINTER_EVENTS.LEAVE, this.handlers.mouseleave)
					 .off(POINTER_EVENTS.MOVE, this.handlers.mousemove)
					 .off(POINTER_EVENTS.DLBCLICK, this.handlers.dblclick)
					 .off(POINTER_EVENTS.UP, this.handlers.mouseup)
					 .off(POINTER_EVENTS.DOWN, this.handlers.mousedown);
	 

				 if(this._initCursor){
					 this.targetNode.style.cursor = this._initCursor.target;
				 }
	 
				 this.handlers = null;
				 if(this.$cursor){
					 this.$cursor.remove();
					 this.$cursor = null;
				 }
	 
				 if(this.$svg){
					 this.$svg.hide(HIDE_DURATION, function(){ $(this).remove(); });
					 this.$svg = null;
					 this.lastLine = null;
				 }
	 
				 BaseView.inherited('remove', this, arguments);
			 },
	 
			 /**
					  * Returns the co-ordinate of the polygon (square) forming the lasso selection
					  */
			 _getCords: function(){
				 return this._points;
			 },
	 
			 /**
				  *
				  * Pass handler through a pipe
				  *	@param handler - input handler function
					 * @return Handler function which executes logic before calling the input handler.
					 */
			 _getPipedHandler: function(handler){
	 
				 var func = function(event){
	 
					 if(this._isTouch(event)){
						 event.preventDefault();
						 this.cordinateResolver(event);
					 }
					 // Lasso interaction don't deselect the widget
					 var eventChainLocal = new EventChainLocal(event);
					 eventChainLocal.setProperty('preventWidgetDeselect', true);
	 
					 handler.bind(this)(event);
				 };
	 
				 return func.bind(this);
			 },
	 
			 _isTouch: function(event){
				 return event.type && (
					 event.type === 'touchstart' ||
					 event.type === 'touchmove' ||
					 event.type === 'touchend');
			 },
	 
			 _updateCursor: function(event, show){
	 
				 if( this._canAutoClosePolygon(event) ){
					 this.targetNode.style.cursor =  CURSOR.CLOSED;
				 }else{
					 this.targetNode.style.cursor =  CURSOR.DEFAULT;
				 }
	 
				 if(!this._isIE()){
					 return;
				 }
	 
				 const OFFSET = {
					 LEFT: 10,
					 TOP: 12
				 };
	 
				 if(show === false){
					 if(this.$cursor){
						 this.$cursor.hide();
					 }
					 return;
				 }
	 
				 if(this.$cursor && !this.$cursor.is(':visible')){
					 this.$cursor.show();
				 }else if(!this.$cursor){
					 this.$cursor = $('<div class="lassoIconBox">');
					 this.$cursor.prependTo(document.body);
				 }
				 this.$cursor.css({ left: event.pageX + OFFSET.LEFT , top: event.pageY + OFFSET.TOP });
			 },
	 
			 _isIE: function(){
				 return BrowserUtils.isIE();
			 },
	 
			 _configCursor: function(){
				 this._initCursor = {};
				 this._initCursor.target = this.targetNode.style.cursor;
				 this.targetNode.style.cursor = CURSOR.default;
			 },
	 
			 _canAutoClosePolygon: function(event){
				 const distance = this._distance(this._points[0], { x: event.clientX, y: event.clientY });
				 return this._points.length > MIN_NUMBER_OF_POINTS && distance !== -1 && distance <= MIN_DISTANCE;
			 },
	 
			 getCoordinates: function(){}
		 });
	 
		 return LassoSelectView;
	 });