/**
 * Licensed Materials - Property of IBM
 *
 * IBM Cognos Products: BI
 *
 * Copyright IBM Corp. 2020
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/**
 	* @file Lasso selection view
 	*/

	 'use strict';
	 define([
		 'jquery',
		 'dashboard-core/js/lib/@waca/core-client/js/core-client/ui/View',
		 'dashboard-core/js/lib/@waca/core-client/js/core-client/utils/BrowserUtils',
		 '@waca/dashboard-common/js/utils/EventChainLocal'
	 ],function($, BaseView, BrowserUtils, EventChainLocal) {
	 
		 const keyCodes = {
			 LEFT_CLICK:1,
			 ESCAPE: 27
		 };
	 
		 const POINTER_EVENTS = {
			 DOWN: 'mousedown touchstart',
			 MOVE: 'mousemove touchmove',
			 UP: 'mouseup touchend',
			 LEAVE: 'mouseleave'
		 };
	 
		 const EXTENSION_ID = 'com.ibm.bi.dashboard.live-lassoView.radius';
	 
	 
		 var LassoSelectView = BaseView.extend({
	 
			 id: EXTENSION_ID,
	 
			 /**
			  * @param {options} set of initial properties
			  *
			  */
			 init: function($target, cordinateResolver, isDragging) {
				 BaseView.inherited('init', this, arguments);
				 this.$target = $target;
				 this.targetNode = this.$target.get(0);
				 this.$body = $(document.body);
				 this.cordinateResolver = cordinateResolver;
				 this._isDragging = isDragging;
	 
				 this.handlers = {
					 mousemove: this._getPipedHandler(this._handleMouseMove),
					 mouseup: this._getPipedHandler(this._handleMouseUp),
					 mousedown: this._getPipedHandler(this._handleMouseDown),
					 keyup: this._getPipedHandler(this._handleKeyup),
					 mouseleave: this._getPipedHandler(this._handleMouseLeave)
				 };
	 
				 if(!this._isIE()){
					 this._configCursor();
				 }
	 
				 this.$el.on(POINTER_EVENTS.MOVE, this.handlers.mousemove);
				 this.$body.keyup(this.handlers.keyup);
				 this.$target.on(POINTER_EVENTS.LEAVE, this.handlers.mouseleave)
					 .on(POINTER_EVENTS.MOVE, this.handlers.mousemove)
					 .on(POINTER_EVENTS.DOWN, this.handlers.mousedown);
	 
				 this._center = null;
			 },
	 
			 /**
			  * Mouse move event handler
			  * @param {Object} event Jquery event object
			  */
			 _handleMouseMove: function(event){
	 
				 if(!this._center){
					 return;
				 }
	 
				 this._updateCursor(event);
	 
				 if(this._isDragging(event)){
					 const rect  = this.targetNode.getBoundingClientRect();
					 const top = rect.top;
					 const left = rect.left;
					 const radius = this._distance(this._center,  { x: event.clientX - left, y: event.clientY - top });
					 this.$svg.children()[0].setAttribute( 'r', radius );
					 this.trigger('lassoSelect:cordsChange', { event: this._centerEvent, radius: radius });
				 }
			 },
	 
			 /**
			  * Handle key up event
			  * @param {Object} event Jquery event object
			  */
			 _handleKeyup: function(e){
				 if(e.keyCode === keyCodes.ESCAPE){
					 this.trigger('lassoSelect:cancel', {});
				 }
			 },
	 
			 /**
			  * Mouse up event handler
			  * @param {Object} event Jquery event object
			  */
			 _handleMouseUp: function(event){
	 
				 const rect  = this.targetNode.getBoundingClientRect();
				 const top = rect.top;
				 const left = rect.left;
				 const radius = this._distance(this._center, { x: event.clientX - left, y: event.clientY - top });
				 this.trigger('lassoSelect:done', { event: this._centerEvent, radius: radius });
	 
				 this._center = null;
				 this._centerEvent = null;
				 if(this.$svg){
					 this.$svg.remove();
					 this.$svg = null;
				 }
				 this.$body.off(POINTER_EVENTS.UP, this.handlers.mouseup);
			 },
	 
			 /**
			  * Mouse down event handler
			  * @param {Object} event Jquery event object
			  */
			 _handleMouseDown: function(event){
	 
				 if(this.$svg){
					 this.$svg.remove();
				 }
	 
				 if(event.type === 'touchstart' || event.which === keyCodes.LEFT_CLICK){
	 
					 const rect  = this.targetNode.getBoundingClientRect();
					 const top = rect.top;
					 const left = rect.left;
					 const width = rect.width;
					 const height = rect.height;
	 
					 const circle = document.createElement('circle');
					 this._center = { x: event.clientX - left,  y: event.clientY - top };
					 this._centerEvent = event;
					 const svg = document.createElement('svg');
	 
					 svg.setAttribute('id', this.id);
					 $(svg).css({ left:left, top:top, width:width, height:height });
	 
					 const attributes = { cx: this._center.x, cy: this._center.y };
					 Object.keys(attributes).forEach(function(name){
						  circle.setAttribute(name, attributes[name])
					 });
					 svg.appendChild(circle);
	 
					 this.$body.prepend(svg.outerHTML);
					 this.$svg = $(document.getElementById(this.id));
	 
					 this.$body.on(POINTER_EVENTS.UP, this.handlers.mouseup);
				 }
			 },
	 
			 _handleMouseLeave: function(event){
				 this._updateCursor(event, false);
			 },
	 
			 remove: function(){
	 
				 this.$body.off(POINTER_EVENTS.UP, this.handlers.mouseup)
					 .off('keyup', this.handlers.keyup);
	 
				 this.$target.off(POINTER_EVENTS.DOWN, this.handlers.mousedown)
					 .off(POINTER_EVENTS.LEAVE, this.handlers.mouseleave)
					 .off(POINTER_EVENTS.MOVE, this.handlers.mousemove);
	 
				 if(this.$svg){
					 this.$svg.remove();
					 this.$svg = null;
				 }
	 
				 if(this._initCursor){
					 this.targetNode.style.cursor = this._initCursor.target;
				 }
	 
				 this.handlers = null;
				 if(this.$cursor){
					 this.$cursor.remove();
					 this.$cursor = null;
				 }
				 BaseView.inherited('remove', this, arguments);
			 },
	 
			 /**
		  *
		  * Pass handler through a pipe
		  *	@param handler - input handler function
			 * @return Handler function which executes logic before calling the input handler.
			 */
			 _getPipedHandler: function(handler){
	 
				 var func = function(event){
	 
					 if(this._isTouch(event)){
						 event.preventDefault();
						 this.cordinateResolver(event);
					 }
					 // Lasso interaction don't deselect the widget
					 var eventChainLocal = new EventChainLocal(event);
					 eventChainLocal.setProperty('preventWidgetDeselect', true);
	 
					 handler.bind(this)(event);
				 };
	 
				 return func.bind(this);
			 },
	 
			 _isTouch: function(event){
				 return event.type && (
					 event.type === 'touchstart' ||
					 event.type === 'touchmove' ||
					 event.type === 'touchend');
			 },
	 
			 _updateCursor: function(event, show){
	 
				 if(!this._isIE()){
					 return;
				 }
	 
				 const OFFSET = {
					 LEFT: 10,
					 TOP: 12
				 };
	 
				 if(show === false){
					 if(this.$cursor){
						 this.$cursor.hide();
					 }
					 return;
				 }
	 
				 if(this.$cursor && !this.$cursor.is(':visible')){
					 this.$cursor.show();
				 }else if(!this.$cursor){
					 this.$cursor = $('<div class="lassoIconBox">');
					 this.$cursor.prependTo(document.body);
				 }
				 this.$cursor.css({ left: event.pageX + OFFSET.LEFT , top: event.pageY + OFFSET.TOP });
			 },
	 
			 _isIE: function(){
				 return BrowserUtils.isIE();
			 },
	 
			 _configCursor: function(){
				 var curVal = 'url(dashboard-analytics/images/circle-dash.svg), auto';
				 this._initCursor = {};
				 this._initCursor.target = this.targetNode.style.cursor;
				 this.targetNode.style.cursor = curVal;
			 },
	 
			 _distance: function(pointA, pointB){
				 if(!pointA || !pointB){
					 return -1;
				 }
	 
				 const deltaX = pointB.x - pointA.x;
				 const deltaY = pointB.y - pointA.y;
				 return Math.sqrt( deltaX * deltaX + deltaY * deltaY );
			 },
	 
			 getCoordinates: function(){} //TODO check case
		 });
	 
		 return LassoSelectView;
	 });